@@ -16,14 +16,13 @@
 use std::fmt::{Debug, Formatter, self};
 use std::collections::{HashMap, HashSet};
 use syn::{
-    Type, Path, Attribute, Meta, MetaList, MetaNameValue, NestedMeta, Lit, LitInt, LitStr,
-    LitByteStr, LitFloat, LitChar, LitBool,
+    Type, Path, Attribute, Meta, MetaNameValue, Lit, LitInt, LitStr, LitByteStr, LitFloat, LitChar,
+    LitBool,
 };
-use syn::parse::{Parse, Parser};
-use proc_macro::TokenStream;
-use quote::ToTokens;
+use syn::parse_quote::ParseQuote;
+use syn::parse::Parser;
 
-use crate::{Error, ArgValue, ArgValueReq, AttrReq, MetaArgs, MetaArgNameValue};
+use crate::{Error, ArgValue, ArgValueReq, AttrReq, MetaArg, MetaArgNameValue, MetaArgList};
 
 /// Internal structure representation of a proc macro attribute collected
 /// instances having some specific name (accessible via [`Attr::name()`]).
@@ -51,11 +50,11 @@ pub enum Attr {
 /// enum.
 ///
 /// Internally the structure is composed of the `name` and `value` fields,
-/// where name is always a [`Ident`] (corresponding `name1`, `name2`, `name3`
-/// from the sample above) and `value` is an optional literal [`Lit`], with
-/// corresponding cases of `None`,
-/// `Some(`[`AttrArgValue::Lit`]`(`[`Lit::Str`]`(`[`LitStr`]`)))`, and
-/// `Some(`[`AttrArgValue::Type`]`(`[`Type::Path`]`(`[`Path`]`)))`.
+/// where name is always a [`struct@syn::Ident`] (corresponding `name1`,
+/// `name2`, `name3` from the sample above) and `value` is an optional literal
+/// [`enum@Lit`], with corresponding cases of `None`,
+/// `Some(`[`ArgValue::Literal`]`(`[`Lit::Str`]`(`[`struct@LitStr`]`)))`, and
+/// `Some(`[`ArgValue::Type`]`(`[`Type::Path`]`(`[`Path`]`)))`.
 #[derive(Clone, Debug)]
 pub struct SingularAttr {
     /// Optional attribute argument path part; for instance in
@@ -91,8 +90,9 @@ pub struct ParametrizedAttr {
     /// field by default, even if they do not represent any known rust path or
     /// type name, like `#[attr(some_id, other)]`. However, with
     /// [`ParametrizedAttr::check`] and [`ParametrizedAttr::checked`] those
-    /// values matching ones specified in [`AttrReq::args`] with values set to
-    /// [`ValueOccurrences::Default`] are moved into [`ParametrizedAttr::args`].
+    /// values matching ones specified in [`AttrReq::arg_req`] with values set
+    /// to [`crate::ListReq::Predefined::default`] are moved into
+    /// [`ParametrizedAttr::args`].
     pub paths: Vec<Path>,
 
     /// Unnamed string literal found within attribute arguments.
@@ -127,10 +127,10 @@ pub struct ParametrizedAttr {
 impl Attr {
     /// Constructs [`Attr`] from a vector of all syn-parsed attributes,
     /// selecting attributes matching the provided name.
-    pub fn with(name: impl ToString + AsRef<str>, attrs: &Vec<Attribute>) -> Result<Self, Error> {
+    pub fn with(name: impl ToString + AsRef<str>, attrs: &[Attribute]) -> Result<Self, Error> {
         SingularAttr::with(name.to_string(), attrs)
-            .map(|singular| Attr::Singular(singular))
-            .or_else(|_| ParametrizedAttr::with(name, attrs).map(|param| Attr::Parametrized(param)))
+            .map(Attr::Singular)
+            .or_else(|_| ParametrizedAttr::with(name, attrs).map(Attr::Parametrized))
     }
 
     /// Constructor parsing [`Attribute`] value and returning either
@@ -140,13 +140,11 @@ impl Attr {
     /// If the attribute does not match either of forms, a [`Error`] is
     /// returned. Currently, only single type of error may occur in practice:
     /// - [`Error::ArgNameMustBeIdent`], which happens if the attribute name is
-    ///   not an [`Ident`] but is a complex path value
+    ///   not an [`struct@syn::Ident`] but is a complex path value
     pub fn from_attribute(attr: &Attribute) -> Result<Self, Error> {
         SingularAttr::from_attribute(attr)
-            .map(|singular| Attr::Singular(singular))
-            .or_else(|_| {
-                ParametrizedAttr::from_attribute(attr).map(|param| Attr::Parametrized(param))
-            })
+            .map(Attr::Singular)
+            .or_else(|_| ParametrizedAttr::from_attribute(attr).map(Attr::Parametrized))
     }
 
     /// Returns inner value \in form of [`SingularAttr`] for [`Attr::Singular`]
@@ -220,7 +218,7 @@ impl SingularAttr {
     /// selecting single attribute matching the provided name. If there are
     /// multiple instances of the same attribute, fails with
     /// [`Error::SingularAttrRequired`]
-    pub fn with(name: impl ToString, attrs: &Vec<Attribute>) -> Result<Self, Error> {
+    pub fn with(name: impl ToString, attrs: &[Attribute]) -> Result<Self, Error> {
         let name = name.to_string();
         let mut filtered_attrs = attrs.iter().filter(|attr| attr.path.is_ident(&name));
         let res = if let Some(attr) = filtered_attrs.next() {
@@ -301,7 +299,7 @@ impl SingularAttr {
     ///   `other` has a named argument with the same name but different values.
     pub fn merge(&mut self, other: Self) -> Result<(), Error> {
         if self.name != other.name {
-            return Err(Error::NamesDontMatch(self.name.clone(), other.name.clone()));
+            return Err(Error::NamesDontMatch(self.name.clone(), other.name));
         }
         match (&self.value, &other.value) {
             (_, ArgValue::None) => {}
@@ -345,7 +343,7 @@ impl SingularAttr {
     }
 
     /// Checks that the structure meets provided value requirements (see
-    /// [`ValueReq`]), generating [`Error`] if the requirements are not met.
+    /// [`ArgValueReq`]), generating [`Error`] if the requirements are not met.
     pub fn check(&mut self, req: ArgValueReq) -> Result<(), Error> {
         req.check(&mut self.value, &self.name, &self.name)?;
         Ok(())
@@ -380,18 +378,10 @@ impl ParametrizedAttr {
 
     /// Constructs [`ParametrizedAttr`] from a vector of all syn-parsed
     /// attributes, selecting attributes matching the provided name.
-    pub fn with(name: impl ToString + AsRef<str>, attrs: &Vec<Attribute>) -> Result<Self, Error> {
+    pub fn with(name: impl ToString + AsRef<str>, attrs: &[Attribute]) -> Result<Self, Error> {
         let mut me = ParametrizedAttr::new(name.to_string());
         for attr in attrs.iter().filter(|attr| attr.path.is_ident(&name)) {
-            match attr.parse_meta()? {
-                // `#[ident(...)]`
-                Meta::List(MetaList { nested, .. }) => {
-                    for meta in nested {
-                        me.fuse(meta)?;
-                    }
-                }
-                _ => return Err(Error::ParametrizedAttrRequired(name.to_string())),
-            }
+            me.fuse(attr)?;
         }
         Ok(me)
     }
@@ -403,15 +393,7 @@ impl ParametrizedAttr {
             .get_ident()
             .ok_or(Error::ArgNameMustBeIdent)?
             .to_string();
-        match attr.parse_meta()? {
-            // `#[ident(...)]`
-            Meta::List(MetaList { nested, .. }) => nested
-                .into_iter()
-                .fold(Ok(ParametrizedAttr::new(name)), |res, nested| {
-                    res.and_then(|attr| attr.fused(nested))
-                }),
-            _ => Err(Error::ParametrizedAttrRequired(name)),
-        }
+        ParametrizedAttr::new(name).fused(attr)
     }
 
     /// Returns literal value for a given argument with name `name`, if it is
@@ -434,10 +416,7 @@ impl ParametrizedAttr {
     /// verbatim2]`, i.e. path arguments containing single path segment and no
     /// value or nested arguments.
     pub fn has_verbatim(&self, verbatim: &str) -> bool {
-        self.paths
-            .iter()
-            .find(|path| path.is_ident(verbatim))
-            .is_some()
+        self.paths.iter().any(|path| path.is_ident(verbatim))
     }
 
     /// Returns set of verbatim attribute arguments.
@@ -463,7 +442,7 @@ impl ParametrizedAttr {
     ///   `other` has a literals which values are not equal.
     pub fn merge(&mut self, other: Self) -> Result<(), Error> {
         if self.name != other.name {
-            return Err(Error::NamesDontMatch(self.name.clone(), other.name.clone()));
+            return Err(Error::NamesDontMatch(self.name.clone(), other.name));
         }
 
         self.args.extend(other.args);
@@ -530,73 +509,75 @@ impl ParametrizedAttr {
         Ok(self)
     }
 
-    /// Fuses data from a nested attribute arguments (see [`syn::NestedMeta`])
-    /// into the attribute parameters.
+    /// Fuses data from a nested attribute arguments (see [`Attribute`]) into
+    /// the attribute parameters.
     ///
     /// The operation is similar to the [`ParametrizedAttr::enrich`] with the
     /// only difference that enrichment operation takes the whole attribute, and
     /// fusion takes a nested meta data.
     #[inline]
-    pub fn fuse(&mut self, nested: NestedMeta) -> Result<(), Error> {
-        let nested = MetaArgs::parse.parse(TokenStream::from(nested.to_token_stream()))?;
-        match nested {
-            // `#[ident("literal", ...)]`
-            MetaArgs::Literal(Lit::Str(s)) => {
-                let span = s.span();
-                match self.string {
-                    None => self.string = Some(s),
-                    Some(ref mut str1) => {
-                        let mut joined = str1.value();
-                        joined.push_str(&s.value());
-                        *str1 = LitStr::new(&joined, span);
+    pub fn fuse(&mut self, attr: &Attribute) -> Result<(), Error> {
+        let args = MetaArgList::parse.parse(attr.tokens.clone().into())?;
+        for arg in args.list {
+            match arg {
+                // `#[ident("literal", ...)]`
+                MetaArg::Literal(Lit::Str(s)) => {
+                    let span = s.span();
+                    match self.string {
+                        None => self.string = Some(s),
+                        Some(ref mut str1) => {
+                            let mut joined = str1.value();
+                            joined.push_str(&s.value());
+                            *str1 = LitStr::new(&joined, span);
+                        }
                     }
                 }
-            }
 
-            // `#[ident(b"literal", ...)]`
-            MetaArgs::Literal(Lit::ByteStr(s)) => {
-                let span = s.span();
-                match self.bytes {
-                    None => self.bytes = Some(s),
-                    Some(ref mut str1) => {
-                        let mut joined = str1.value();
-                        joined.extend(&s.value());
-                        *str1 = LitByteStr::new(&joined, span);
+                // `#[ident(b"literal", ...)]`
+                MetaArg::Literal(Lit::ByteStr(s)) => {
+                    let span = s.span();
+                    match self.bytes {
+                        None => self.bytes = Some(s),
+                        Some(ref mut str1) => {
+                            let mut joined = str1.value();
+                            joined.extend(&s.value());
+                            *str1 = LitByteStr::new(&joined, span);
+                        }
                     }
                 }
-            }
 
-            // `#[ident(3, ...)]`
-            MetaArgs::Literal(Lit::Int(lit)) => self.integers.push(lit),
+                // `#[ident(3, ...)]`
+                MetaArg::Literal(Lit::Int(lit)) => self.integers.push(lit),
 
-            // `#[ident(2.3, ...)]`
-            MetaArgs::Literal(Lit::Float(lit)) => self.floats.push(lit),
+                // `#[ident(2.3, ...)]`
+                MetaArg::Literal(Lit::Float(lit)) => self.floats.push(lit),
 
-            // `#[ident('a', ...)]`
-            MetaArgs::Literal(Lit::Char(lit)) => self.chars.push(lit),
+                // `#[ident('a', ...)]`
+                MetaArg::Literal(Lit::Char(lit)) => self.chars.push(lit),
 
-            // `#[ident(true, ...)]`
-            MetaArgs::Literal(Lit::Bool(_)) if self.bool.is_some() => {
-                return Err(Error::MultipleLiteralValues(self.name.clone()))
-            }
-            MetaArgs::Literal(Lit::Bool(lit)) if self.bool.is_none() => {
-                self.bool = Some(lit.clone())
-            }
+                // `#[ident(true, ...)]`
+                MetaArg::Literal(Lit::Bool(_)) if self.bool.is_some() => {
+                    return Err(Error::MultipleLiteralValues(self.name.clone()))
+                }
+                MetaArg::Literal(Lit::Bool(ref lit)) if self.bool.is_none() => {
+                    self.bool = Some(lit.clone())
+                }
 
-            // `#[ident(true, ...)]`
-            MetaArgs::Literal(_) => return Err(Error::UnsupportedLiteral(self.name.clone())),
+                // `#[ident(true, ...)]`
+                MetaArg::Literal(_) => return Err(Error::UnsupportedLiteral(self.name.clone())),
 
-            // `#[ident(arg::path)]`
-            MetaArgs::Path(path) => self.paths.push(path),
+                // `#[ident(arg::path)]`
+                MetaArg::Path(path) => self.paths.push(path),
 
-            // `#[ident(name = value, ...)]`
-            MetaArgs::NameValue(MetaArgNameValue { name, value, .. }) => {
-                let id = name.to_string();
-                if self.args.insert(id.clone(), value).is_some() {
-                    return Err(Error::ArgNameMustBeUnique {
-                        attr: self.name.clone(),
-                        arg: id,
-                    });
+                // `#[ident(name = value, ...)]`
+                MetaArg::NameValue(MetaArgNameValue { name, value, .. }) => {
+                    let id = name.to_string();
+                    if self.args.insert(id.clone(), value).is_some() {
+                        return Err(Error::ArgNameMustBeUnique {
+                            attr: self.name.clone(),
+                            arg: id,
+                        });
+                    }
                 }
             }
         }
@@ -607,8 +588,8 @@ impl ParametrizedAttr {
     /// it consumes the self and returns an enriched structure in case of
     /// the successful operation. Useful in operation chains.
     #[inline]
-    pub fn fused(mut self, nested: NestedMeta) -> Result<Self, Error> {
-        self.fuse(nested)?;
+    pub fn fused(mut self, attr: &Attribute) -> Result<Self, Error> {
+        self.fuse(attr)?;
         Ok(self)
     }
 
@@ -618,17 +599,23 @@ impl ParametrizedAttr {
     /// The procedure modifies the [`ParametrizedAttr`] data in the following
     /// ways:
     /// 1. First, it fills in [`ParametrizedAttr::paths`],
-    ///    [`ParametrizedAttr::integers`] and [`ParametrizedAttr::literal`] with
-    ///    default values    from [`AttrReq::paths`], [`AttrReq::integers`] and
-    ///    [`AttrReq::literal`] (correspondingly).
-    /// 2. [`ParametrizedAttr::paths`] values matching ones specified in
-    ///    [`AttrReq::args`] with values set to [`ValueOccurrences::Default`]
-    ///    are moved into [`ParametrizedAttr::args`] field.
+    ///    [`ParametrizedAttr::integers`], [`ParametrizedAttr::floats`],
+    ///    [`ParametrizedAttr::chars`], [`ParametrizedAttr::bytes`],
+    ///    [`ParametrizedAttr::string`] with default values from
+    ///    [`AttrReq::path_req`], [`AttrReq::integer_req`],
+    ///    [`AttrReq::float_req`], [`AttrReq::char_req`],
+    ///    [`AttrReq::bytes_req`], [`AttrReq::string_req`] correspondingly.
+    /// 2. [`ParametrizedAttr::paths`] values
+    ///    matching ones specified in [`AttrReq::arg_req`] with values set to
+    ///    [`crate::ListReq::Predefined::default`] are moved into
+    ///    [`ParametrizedAttr::args`] field.
     pub fn check(&mut self, req: AttrReq) -> Result<(), Error> {
         for (name, req) in &req.arg_req {
             if let Some(pos) = self.paths.iter().position(|path| path.is_ident(name)) {
                 self.paths.remove(pos);
-                self.args.entry(name.clone()).or_insert(req.default_value());
+                self.args
+                    .entry(name.clone())
+                    .or_insert_with(|| req.default_value());
             }
 
             if !self.args.contains_key(name) && req.is_required() {
@@ -763,7 +750,7 @@ impl Debug for ParametrizedAttr {
             f.write_str("\n\t")?;
         }
 
-        write!(f, "name: {:?},", self.name)?;
+        write!(f, "name: {:?}, ", self.name)?;
         if f.alternate() {
             f.write_str("\n\t")?;
         }
@@ -777,7 +764,7 @@ impl Debug for ParametrizedAttr {
                 if f.alternate() {
                     f.write_str("\t\t")?;
                 }
-                write!(f, "{} => {:?},", name, val)?;
+                write!(f, "{} => {:?}, ", name, val)?;
                 if f.alternate() {
                     f.write_str("\n")?;
                 }
@@ -786,7 +773,7 @@ impl Debug for ParametrizedAttr {
                 f.write_str("\t")?;
             }
         }
-        f.write_str("},")?;
+        f.write_str("}, ")?;
         if f.alternate() {
             f.write_str("\n\t")?;
         }
@@ -800,7 +787,7 @@ impl Debug for ParametrizedAttr {
                 if f.alternate() {
                     f.write_str("\t\t")?;
                 }
-                write!(f, "{},", quote! { #path })?;
+                write!(f, "{}, ", quote! { #path })?;
                 if f.alternate() {
                     f.write_str("\n")?;
                 }
@@ -809,18 +796,18 @@ impl Debug for ParametrizedAttr {
                 f.write_str("\t")?;
             }
         }
-        f.write_str("],")?;
+        f.write_str("], ")?;
         if f.alternate() {
             f.write_str("\n\t")?;
         }
 
         write!(
             f,
-            "bool: {:?},",
+            "bool: {:?}, ",
             self.bool
                 .as_ref()
                 .map(|b| format!("Some({:?})", b.value))
-                .unwrap_or("None".to_owned())
+                .unwrap_or_else(|| "None".to_owned())
         )?;
         if f.alternate() {
             f.write_str("\n\t")?;
@@ -828,11 +815,11 @@ impl Debug for ParametrizedAttr {
 
         write!(
             f,
-            "string: {:?},",
+            "string: {:?}, ",
             self.string
                 .as_ref()
                 .map(|s| format!("Some({:?})", s.value()))
-                .unwrap_or("None".to_owned())
+                .unwrap_or_else(|| "None".to_owned())
         )?;
         if f.alternate() {
             f.write_str("\n\t")?;
@@ -840,11 +827,11 @@ impl Debug for ParametrizedAttr {
 
         write!(
             f,
-            "bytes: {:?}",
+            "bytes: {:?}, ",
             self.bytes
                 .as_ref()
                 .map(|s| format!("Some({:?})", s.value()))
-                .unwrap_or("None".to_owned())
+                .unwrap_or_else(|| "None".to_owned())
         )?;
         if f.alternate() {
             f.write_str("\n\t")?;
@@ -859,7 +846,7 @@ impl Debug for ParametrizedAttr {
                 if f.alternate() {
                     f.write_str("\t\t")?;
                 }
-                write!(f, "{},", quote! { #c })?;
+                write!(f, "{}, ", quote! { #c })?;
                 if f.alternate() {
                     f.write_str("\n")?;
                 }
@@ -868,7 +855,7 @@ impl Debug for ParametrizedAttr {
                 f.write_str("\t")?;
             }
         }
-        f.write_str("],")?;
+        f.write_str("], ")?;
         if f.alternate() {
             f.write_str("\n\t")?;
         }
@@ -882,7 +869,7 @@ impl Debug for ParametrizedAttr {
                 if f.alternate() {
                     f.write_str("\t\t")?;
                 }
-                write!(f, "{},", quote! { #c })?;
+                write!(f, "{}, ", quote! { #c })?;
                 if f.alternate() {
                     f.write_str("\n")?;
                 }
@@ -891,7 +878,7 @@ impl Debug for ParametrizedAttr {
                 f.write_str("\t")?;
             }
         }
-        f.write_str("],")?;
+        f.write_str("], ")?;
         if f.alternate() {
             f.write_str("\n\t")?;
         }
@@ -905,7 +892,7 @@ impl Debug for ParametrizedAttr {
                 if f.alternate() {
                     f.write_str("\t\t")?;
                 }
-                write!(f, "{},", quote! { #c })?;
+                write!(f, "{}, ", quote! { #c })?;
                 if f.alternate() {
                     f.write_str("\n")?;
                 }
@@ -914,7 +901,7 @@ impl Debug for ParametrizedAttr {
                 f.write_str("\t")?;
             }
         }
-        f.write_str("],")?;
+        f.write_str("], ")?;
 
         if f.alternate() {
             f.write_str("\n")?;
